--
--	Package File Template
--
--	Purpose: This package defines supplemental types, subtypes, 
--		 constants, and functions 
--
--   To use any of the example code shown below, uncomment the lines and modify as necessary
--

library IEEE;
use IEEE.STD_LOGIC_1164.all;

use IEEE.NUMERIC_STD.ALL;


package common is

	type board_type_t is (ofcuros, tscrear);
	constant board_type : board_type_t := tscrear;
	
	constant exclude1wire : Boolean := false;

	constant build_num : std_logic_vector := "0000101";
	
	type board2bit_t is array(board_type_t) of std_logic;
	type board2byte_t is array(board_type_t) of std_logic_vector(7 downto 0);
	constant isofcu2byte : board2byte_t := (ofcuros => (others => '1'), tscrear => (others => '0'));
	constant isofcu2bit : board2bit_t := (ofcuros => '1', tscrear => '0');


	constant fresh_bits : integer := 13; -- bits of a counter that does a reset in the error flags of both cards
--********************************************************************************
--	<REGISTERS>
--********************************************************************************

	type reg_t is array(natural range <>) of std_logic_vector(7 downto 0);

	constant num_of_regs : integer := 32;

	constant moregs_data			: integer := 0;
	constant moregs_ctrl			: integer := 1;
	--NOTE: owX_write and owX_read share the same register now, which is called owX_data in the documentation
	constant ow1_ctrl_reg 		: integer := 2;	-- One wire - 1.2 commands: (--,--,--,byte read, byte write, bit read, bit write, init)
	constant ow1_write_reg 		: integer := 3;	-- One wire - 1.2 write data
	constant ow1_read_reg 		: integer := 3;	-- One wire - 1.2 read data
	constant ow1_crc_reg 		: integer := 4;	-- One wire - 1.2 CRC
	constant ow3_ctrl_reg 		: integer := 5;	-- One wire - 3.3 commands: (--,--,--,byte read, byte write, bit read, bit write, init)
	constant ow3_write_reg 		: integer := 6;	-- One wire - 3.3 write data
	constant ow3_read_reg 		: integer := 6;	-- One wire - 3.3 read data
	constant ow3_crc_reg 		: integer := 7;	-- One wire - 3.3 CRC
	
	constant sd_status_reg 		: integer := 8;	-- (sd_ch24_l, sd_tsc_l, sd2_l, sd1_l, sd_ch24, sd_tsc, sd2, sd1); wr resets latches
	constant opto_ctrl_reg 		: integer := 9;	-- (tx1_dis, tx2_dis, rx_en2, rx_en1, sq_en2, sq_en1, en_sd2, en_sd1)

	constant tsc_status_0_la	: integer := 10;	-- WR: Simulate ECR // RD: Flags for errors in the tsc channel receiver
	constant tsc_status_1_la	: integer := 11;	-- WR: clear latches// RD: Flags for errors in the tsc channel receiver
	constant tsc_status_2_la	: integer := 12;	-- WR: resets iserdes//Flags for errors in the tsc channel receiver
	constant test0_reg			: integer := 13;	-- Reads memmory of received words; wr advances
	constant test1_reg			: integer := 14;	-- wr forces start storing words again
	constant test2_reg			: integer := 15;	-- wr resets latches and adds 1 to iodelay2

	constant tsc_maxwords		: integer := 16;
	constant tsc_strobewait		: integer := 17;
	
	constant tsc_ecrwait			: integer := 18;
	constant tsc_roswait1		: integer := 19;
	constant tsc_headerwait		: integer := 20;
	constant tsc_datawait		: integer := 21;
	constant tsc_roswait2		: integer := 22;

	constant buildn_reg			: integer := 23;
	constant log2_recaltime_reg: integer := 24;
	constant log2_integr_reg	: integer := 25;
	
	constant tsc_resyncwait		: integer := 26;
	constant tsc_resyncgap		: integer := 27;
	constant tsc_unlocking		: integer := 28;
	
	constant tsc_safetylatency : integer := 29;
	

	

	constant reg_init : reg_t(num_of_regs-1 downto 0) := (
		moregs_data 		=> (others => '0'),
      moregs_ctrl 		=> (others => '0'),
		ow1_ctrl_reg		=> (others => '0'),
		ow1_write_reg 		=> (others => '0'),
		ow1_crc_reg 		=> (others => '0'),
		ow3_ctrl_reg 		=> (others => '0'),
		ow3_write_reg 		=> (others => '0'),
		ow3_crc_reg 		=> (others => '0'),
		sd_status_reg 		=> (others => '1'),
		opto_ctrl_reg		=> isofcu2byte(board_type),
      tsc_status_0_la	=> (others => '0'),
      tsc_status_1_la	=> (others => '0'),
      tsc_status_2_la   => (others => '0'),
		test0_reg			=> (others => '0'),
		test1_reg			=> (others => '0'),
		test2_reg			=> (others => '0'),
		tsc_maxwords		=> std_logic_vector(to_unsigned(80,8)),
		tsc_strobewait		=> std_logic_vector(to_unsigned(20,8)),
		tsc_ecrwait			=> std_logic_vector(to_unsigned(25,8)),
		tsc_roswait1		=> std_logic_vector(to_unsigned(100,8)), -- x64
		tsc_headerwait		=> std_logic_vector(to_unsigned(250,8)),
		tsc_datawait		=> std_logic_vector(to_unsigned(100,8)),
		tsc_roswait2		=> std_logic_vector(to_unsigned(50,8)),
		buildn_reg			=> build_num & isofcu2bit(board_type),
		log2_recaltime_reg=> std_logic_vector(to_unsigned(24,8)), --24 equivale a .84 s
		log2_integr_reg 	=> std_logic_vector(to_unsigned(4,8)), -- 0 = no integrator
		tsc_resyncwait		=> std_logic_vector(to_unsigned(150,8)), -- x64
		tsc_resyncgap		=> std_logic_vector(to_unsigned(100,8)), 
		tsc_unlocking		=> std_logic_vector(to_unsigned(1,8)), 
		tsc_safetylatency	=> std_logic_vector(to_unsigned(30,8)), 
		others 				=> "10101010");

	-- 1: user write/read, 0: read only
	constant reg_mode : std_logic_vector(num_of_regs-1 downto 0) := (
		moregs_data 		=> '1',
      moregs_ctrl 		=> '1',
		ow1_ctrl_reg		=> '1',
		ow1_write_reg 		=> '1',
		ow1_crc_reg 		=> '0',
		ow3_ctrl_reg 		=> '1',
		ow3_write_reg 		=> '1',
		ow3_crc_reg 		=> '0',
		sd_status_reg 		=> '0',
		opto_ctrl_reg		=> '1',
      tsc_status_0_la	=> '0',
		tsc_status_1_la	=> '0',
		tsc_status_2_la   => '0',
		test0_reg 			=> '0',
		test1_reg			=> '0',
		test2_reg			=> '0',
		tsc_maxwords		=>	'1',	
		tsc_strobewait		=> '1',
		tsc_ecrwait			=> '1',
		tsc_roswait1		=> '1',
		tsc_headerwait		=> '1',
		tsc_datawait		=> '1',
		tsc_roswait2		=> '1',
		buildn_reg			=> '0',
		log2_recaltime_reg=> '1',
		log2_integr_reg	=> '1',
		tsc_resyncwait		=>	'1',	
		tsc_resyncgap		=>	'1',	
		tsc_unlocking		=>	'1',	
		tsc_safetylatency	=>	'1',	
		others =>'0');



	type t_registers_i is
	record
		sc_add		: std_logic_vector(3 downto 0);
		sc_bsel		: std_logic;
		sc_wr			: std_logic;
		
		reg_int		: reg_t(num_of_regs-1 downto 0);
		reg_ch_int	: std_logic_vector(num_of_regs-1 downto 0);
	end record;
	
	type t_registers_o is
	record
		reg			: reg_t(num_of_regs-1 downto 0);
		ext_wr		: std_logic_vector(num_of_regs-1 downto 0);
		reg_ch_int	: std_logic_vector(num_of_regs-1 downto 0);
		debug			: std_logic_vector(4 downto 0);
	end record;

	constant t_registers_o_init : t_registers_o := (
		reg 			=> reg_init,
		ext_wr 		=> (others => '0'),
		reg_ch_int 	=> (others => '0'),
		debug 		=> (others => '0')	
	);

	component registers
	Port (
		input 		: in t_registers_i;
		output 		: out t_registers_o;
		sc_data 	: inout std_logic_vector(7 downto 0);
		clk40 	: in std_logic
		);
	end component;

--********************************************************************************
--	</REGISTERS>
--********************************************************************************



--********************************************************************************
--	<ONE WIRE>
--********************************************************************************
	type t_onewire_i is
	record
		reg			: reg_t(num_of_regs-1 downto 0);
		ext_wr		: std_logic_vector(num_of_regs-1 downto 0);
		reg_ch_int	: std_logic_vector(num_of_regs-1 downto 0);
	end record;
	
	type t_onewire_o is
	record
		reg_int		: reg_t(num_of_regs-1 downto 0);
		reg_ch_int	: std_logic_vector(num_of_regs-1 downto 0);
		debug			: std_logic_vector(3 downto 0);
	end record;

	constant t_onewire_o_init : t_onewire_o := (
		reg_int		=> reg_init,
		reg_ch_int	=> (others => '0'),
		debug			=> (others => '0')
	);
	
	
	component onewire
	Generic (
		ctrl_reg		: integer;
		read_reg		: integer;
		write_reg	: integer;
	   crc_reg		: integer
	);
	Port (
		input  		: in t_onewire_i;
		output 		: out t_onewire_o;
		wire		: inout std_logic;
		clk40 	: in std_logic
		);
	end component;
--********************************************************************************
--	</ONE WIRE>
--********************************************************************************




--********************************************************************************
--	<RECEIVERS>
--********************************************************************************
	type t_receivers_i is
	record
		reg			: reg_t(num_of_regs-1 downto 0);
		ext_wr		: std_logic_vector(num_of_regs-1 downto 0);
		reg_ch_int	: std_logic_vector(num_of_regs-1 downto 0);
		
		sd_tsc		: std_logic;
		sd_ch24		: std_logic;
		sd				: std_logic_vector(2 downto 1);
	end record;
	
	type t_receivers_o is
	record
		reg_int		: reg_t(num_of_regs-1 downto 0);
		reg_ch_int	: std_logic_vector(num_of_regs-1 downto 0);
		
		tx_dis		: std_logic_vector(2 downto 1);
		rx_en			: std_logic_vector(2 downto 1);
		sq_en			: std_logic_vector(2 downto 1);
		en_sd			: std_logic_vector(2 downto 1);
	end record;

	constant t_receivers_o_init : t_receivers_o := (
		reg_int		=> reg_init,
		reg_ch_int	=> (others => '0'),
		tx_dis		=> (others => '1'),
		rx_en			=> (others => '0'),
		sq_en			=> (others => '0'),
		en_sd			=> (others => '0')
	);
	
	component receivers
	Port (
		input  		: in t_receivers_i;
		output 		: out t_receivers_o;
		clk40 	: in std_logic
		);
	end component;
--********************************************************************************
--	</RECEIVERS>
--********************************************************************************

--********************************************************************************
--	<SERIALCLK>
--********************************************************************************
	type t_serialclk_i is
	record
		clk40pad				: std_logic;
		reg					: reg_t(num_of_regs-1 downto 0);
	end record;
	
	type t_serialclk_o is
	record
		pll_locked 			: std_logic;
		clk40					: std_logic;
		clk40s				: std_logic;
		clk200 				: std_logic;
		bufpll_locked 		: std_logic_vector(3 downto 0);
		clk800	 			: std_logic_vector(3 downto 0);
		serdesstrobe		: std_logic_vector(3 downto 0);
	end record;

	constant t_serialclk_o_init : t_serialclk_o := (
		pll_locked 			=> '0',
		clk40	 				=> '0',
		clk40s 				=> '0',
		clk200 				=> '0',
		bufpll_locked 		=> (others =>'0'),
		clk800 				=> (others =>'0'),
		serdesstrobe		=> (others =>'0')
	);

	component serialclk
	Port (
		input  		: in t_serialclk_i;
		output 		: out t_serialclk_o;
		reset			: in std_logic
		);
	end component;
--********************************************************************************
--	</SERIALCLK>
--********************************************************************************

--********************************************************************************
--	<DESER>
--********************************************************************************
	constant serialin_bank : integer := 3;

	type t_deser_i is
	record
		serialin_p		: std_logic;
		serialin_n		: std_logic;
		clk200			: std_logic;
		clk800			: std_logic;
		serdesstrobe	: std_logic;
		bufpll_locked	: std_logic;
		reg				: reg_t(num_of_regs-1 downto 0);
		reg_ch_int		: std_logic_vector(num_of_regs-1 downto 0);
	end record;

	type t_deser_o is
	record
		lock 					: std_logic;
		iodelay_at_limit	: std_logic;
		word 					: std_logic_vector(19 downto 0);
		reg_int		: reg_t(num_of_regs-1 downto 0);
		reg_ch_int	: std_logic_vector(num_of_regs-1	 downto 0);
		debug					: std_logic_vector(20 downto 1);

	end record;

	constant t_deser_o_init : t_deser_o := (
		lock 					=> '0',
		iodelay_at_limit	=> '0',
		word 					=> (others => '0'),
		reg_int		=> reg_init,
		reg_ch_int	=> (others => '0'),
		debug					=> (others => '0')
	);
	
	component deser
	Port (
		input  	: in t_deser_i;
		output 	: out t_deser_o;
		reset		: in std_logic;
		clk40 	: in std_logic
		);
	end component;
--********************************************************************************
--	</DESER>
--********************************************************************************

--********************************************************************************
--	<ROSJ3>
--********************************************************************************
	type t_rosj3_i is
	record
      ros_ready			: std_logic;
		evcntres				: std_logic;
		l1a					: std_logic;
		word					: std_logic_vector(19 downto 0);

		reg				: reg_t(num_of_regs-1 downto 0);
		reg_ch_int			: std_logic_vector(num_of_regs-1 downto 0);
		ext_wr				: std_logic_vector(num_of_regs-1 downto 0);
		 --error sources
		bufpll_locked 		: std_logic;
		pll_locked 			: std_logic;
		iodelay_at_limit	: std_logic;
		deser_lock 			: STD_LOGIC;
		ixo_on				: std_logic;
		clk40s				: std_logic;
	end record;
	
	type t_rosj3_o is
	record
		trg_data 	: std_logic_vector(15 downto 0);
      trg_strobe	: std_logic;
		
		reg_int		: reg_t(num_of_regs-1 downto 0);
		reg_ch_int	: std_logic_vector(num_of_regs-1	 downto 0);
		debug					: std_logic_vector(20 downto 1);
	end record;

	constant t_rosj3_o_init : t_rosj3_o := (
		trg_data 	=> (others => '0'),
		trg_strobe	=> '0',
		reg_int		=> reg_init,
		reg_ch_int	=> (others => '0'),
		debug					=> (others => '0')
	);

	component rosj3
	Port (
		input  		: in t_rosj3_i;
		output 		: out t_rosj3_o;
		clk40 		: in std_logic
		);
	end component;
--********************************************************************************
--	</ROSJ3>
--********************************************************************************

-- description of the bits in the 3 tsc-link-status registers
	constant err_bufpll				: integer := 0;
	constant err_pll					: integer := 1; 
	constant err_stayatlimit		: integer := 2; 
	constant err_ixo_lcl 			: integer := 3; 
	constant err_lock 				: integer := 4; 
	constant err_ecr_rmt				: integer := 5; -- enmascarable: ha llegado una palabra de ecr por el enlace serie pero no estaba esperndola; ignorar.
	constant err_ecr_lcl 			: integer := 6; -- enmascarable: ha dado timeout en recibir ecr remoto as que dejo de esperar a que llegue y completo el local
	constant err_fifo 				: integer := 7; 
	constant err_ros_rdy				: integer := 8; -- enmascarable: keeps on waiting... if too much, fifo will be filled and go to error.
	constant err_trg_strobe			: integer := 8; 
	constant err_ros_rdy_fast		: integer := 9; -- currently must go to error to flush the fifo although could be solved
	constant err_ros_rdy_held		: integer := 10;-- enmascarable: can continue waiting. If keeps that way will go to err_fifo
	constant err_hdr_ali				: integer := 11;
	constant err_hdr_bad				: integer := 12;
	constant err_hdr_wait			: integer := 13; 
	constant err_dat_wait			: integer := 14; 
	constant err_dat_sudden_end	: integer := 15; 
	constant err_dat_count_match	: integer := 16;-- enmascarable: doesn't disrupt the state machine
	constant err_dat_count_max		: integer := 17;-- currently must go to error to flush the fifo although could be solved
	constant err_bad_data_trail	: integer := 18;
	constant err_resync				: integer := 19;-- enmascarable: resync unsuccessful, already goes to error
	constant err_inerrorlatch		: integer := 20;
	constant err_resynced			: integer := 21;
	
	constant error_mask 				: std_logic_vector(23 downto 0) := (
		err_ecr_lcl => '0', 
		err_ecr_rmt => '0', 
		err_ros_rdy => '0', 
		err_ros_rdy_held => '0', 
		err_dat_count_match => '0', 
		err_resync  => '0', 
		err_inerrorlatch => '0',
		err_resynced => '0',
		22 => '0',
		others => '1');

--********************************************************************************
--	<TSCJ3>
--********************************************************************************
	type t_tscj3_i is
	record
		trg_data		: std_logic_vector(15 downto 0);
      trg_strobe	: std_logic;
		evcntres		: std_logic;
		l1a			: std_logic;

		reg				: reg_t(num_of_regs-1 downto 0);
		ext_wr		: std_logic_vector(num_of_regs-1 downto 0);
		reg_ch_int			: std_logic_vector(num_of_regs-1 downto 0);
		
		 --error sources
		bufpll_locked 		: std_logic;
		pll_locked 			: std_logic;
		ixo_on		: std_logic;
		clk40s				: std_logic;
	end record;
	
	type t_tscj3_o is
	record
      ros_ready	: std_logic; 
		word			: std_logic_vector(19 downto 0);
		debug			: std_logic_vector(20 downto 1);
		
		reg_int		: reg_t(num_of_regs-1 downto 0);
		reg_ch_int	: std_logic_vector(num_of_regs-1	 downto 0);
	end record;

	constant t_tscj3_o_init : t_tscj3_o := (
		ros_ready	=> '0',
		word			=> (others => '0'),
		debug			=> (others => '0'),
		reg_int		=> reg_init,
		reg_ch_int	=> (others => '0')
	);

	component tscj3
	Port (
		input 	: in t_tscj3_i;
		output	: out t_tscj3_o;
		clk40 	: in std_logic
		);
	end component;
--********************************************************************************
--	</TSCJ3>
--********************************************************************************

--********************************************************************************
--	<SERIALIZER>
--********************************************************************************
	constant serialout_bank : integer := 2;

	type t_serializer_i is
	record
		word			: std_logic_vector(19 downto 0);

		clk200			: std_logic;
		clk800			: std_logic;
		serdesstrobe	: std_logic;
		reg				: reg_t(num_of_regs-1 downto 0);
	end record;

	type t_serializer_o is
	record
		serialout	: std_logic;
		debug					: std_logic_vector(20 downto 1);
	end record;

	constant t_serializer_o_init : t_serializer_o := (
		serialout 	=> '0',
		debug => (others => '0')
	);
	
	component serializer
	Port (
		input  		: in t_serializer_i;
		output 		: out t_serializer_o;
		reset			: in std_logic;
		clk40 		: in std_logic
		);	
	end component;

--********************************************************************************
--	</SERIALIZER>
--********************************************************************************


--********************************************************************************
--	<STATUSLEDS>
--********************************************************************************
	type t_statusleds_i is
	record
		deser_lock		: std_logic;
      sc_bsel_n 		: std_logic;
      evcntres			: std_logic;
		
      l1a				: std_logic;
		reg				: reg_t(num_of_regs-1 downto 0);
	end record;

	type t_statusleds_o is
	record
		leds 				: std_logic_vector(5 downto 0);
	end record;

	constant t_statusleds_o_init : t_statusleds_o := (
		leds 				=> (others => '1')
	);

	
	component statusleds
	Port (
		input  		: in t_statusleds_i;
		output 		: out t_statusleds_o;
		clk40		 	: in std_logic
		);
	end component;
--********************************************************************************
--	</STATUSLEDS>
--********************************************************************************

--********************************************************************************
--	<TEST>
--********************************************************************************
	type t_test_i is
	record
		reg_ch_int			: std_logic_vector(num_of_regs-1 downto 0);
		ext_wr				: std_logic_vector(num_of_regs-1 downto 0);
		reg					: reg_t(num_of_regs-1 downto 0);

		sc_bsel				: std_logic;
		read_mode			: std_logic;
		BSpare				: std_logic_vector(4 downto 0);
		
		deser_lock			: std_logic;
		word					: std_logic_vector(19 downto 0);
		
		clk40s				: std_logic;
	end record;

	type t_test_o is
	record
		word			: std_logic_vector(19 downto 0);
		reg_int		: reg_t(num_of_regs-1 downto 0);
		reg_ch_int	: std_logic_vector(num_of_regs-1	 downto 0);
		lock			: std_logic;
	end record;

	constant t_test_o_init : t_test_o := (
		word			=> (others => '0'),
		reg_int		=> reg_init,
		reg_ch_int	=> (others => '0'),
		lock 			=> '0'
	);

	
	component test
	Port (
		input  		: in t_test_i;
		output 		: out t_test_o;
		testconn		: inout std_logic_vector(20 downto 1);
		reset			: in std_logic;
		clk40		 	: in std_logic
		);
	end component;
--********************************************************************************
--	</TEST>
--********************************************************************************




end common;

package body common is

 
end common;
